import os
import sys
import subprocess
import platform
import shutil
import textwrap
from pathlib import Path
from typing import Optional, Tuple
import google.generativeai as genai

# For Markdown Response Rendering
from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel

# Import readline for better terminal input handling
try:
    import readline
    READLINE_AVAILABLE = True
except ImportError:
    try:
        import pyreadline3 as readline
        READLINE_AVAILABLE = True
    except ImportError:
        print("⚠️  For better terminal experience, install: pip install pyreadline3")
        READLINE_AVAILABLE = False

class AITerminal:
    """An intelligent terminal that uses AI to convert natural language to shell commands."""
    
    def __init__(self):
        # Initialize colors first - this is critical!
        self.colors = {
            'red': '\033[91m',
            'green': '\033[92m',
            'yellow': '\033[93m',
            'blue': '\033[94m',
            'magenta': '\033[95m',
            'cyan': '\033[96m',
            'white': '\033[97m',
            'reset': '\033[0m',
            'bold': '\033[1m',
            'bright_red': '\033[91m\033[1m',
            'bright_green': '\033[92m\033[1m',
            'bright_yellow': '\033[93m\033[1m',
            'bright_blue': '\033[94m\033[1m',
            'bright_magenta': '\033[95m\033[1m',
            'bright_cyan': '\033[96m\033[1m'
        }
        
        # Initialize other attributes
        self.api_key = None
        self.current_dir = Path.cwd()
        self.command_history = []
        self.ask_history = []  # Dedicated history for /ask prompts
        self.max_history = 100
        self.max_ask_history = 50  # Separate limit for ask history
        self.system_info = self._get_system_info()
        self.config_dir = Path.home() / '.commandor'
        self.env_file = self.config_dir / '.env'
        self.ask_history_file = self.config_dir / 'ask_history.txt'
        self.model = None  # Initialize model as None
        
        # Initializing the Rich console
        self.console = Console()
        
        # Ensure config directory exists
        try:
            self.config_dir.mkdir(exist_ok=True)
        except Exception as e:
            print(f"Warning: Could not create config directory: {e}")
        
        # Setup API key
        self._setup_api_key()
        
        # Configure Gemini only if we have an API key
        if self.api_key:
            try:
                genai.configure(api_key=self.api_key)
                self.model = genai.GenerativeModel("gemini-2.0-flash")
            except Exception as e:
                print(f"Warning: Could not initialize Gemini model: {e}")
                self.model = None
        
        # Setup readline if available
        self._setup_readline()
        
        # Load ask history
        self._load_ask_history()
        
    def _display_ai_response(self, response: str, title: str = "AI Response"):
        """Display AI response using rich library for better formatting."""
        
        # Create markdown object
        markdown = Markdown(response)
        
        # Create a panel with the markdown content
        panel = Panel(
            markdown,
            title=f"🤖 {title}",
            subtitle="Powered by Gemini",
            border_style="cyan",
            padding=(1, 2),
            expand=False
        )
        
        self.console.print(panel)

    def reset_api_key(self):
        """Reset and reconfigure the API key."""
        print(self._colorize("🔄 Resetting API key...", 'yellow'))
        self.api_key = None
        self.model = None
        self._prompt_for_api_key()

    def test_api_key(self) -> bool:
        """Test if the current API key is valid."""
        if not self.api_key:
            print(self._colorize("❌ No API key configured", 'red'))
            return False
        
        try:
            print(self._colorize("🔍 Testing API key...", 'yellow'))
            genai.configure(api_key=self.api_key)
            test_model = genai.GenerativeModel("gemini-2.0-flash")
            test_response = test_model.generate_content("Hello")
            return test_response is not None
        except Exception as e:
            print(self._colorize(f"❌ API key test failed: {str(e)}", 'red'))
            return False

    def handle_api_error(self, error_message: str) -> bool:
        """Handle API errors and offer to reset API key."""
        print(self._colorize(f"❌ API Error: {error_message}", 'red'))
        
        # Check if it's likely an API key issue
        api_error_keywords = ['api key', 'authentication', 'unauthorized', 'forbidden', 'invalid', 'quota', 'exceeded']
        if any(keyword in error_message.lower() for keyword in api_error_keywords):
            print(self._colorize("🔍 This looks like an API key issue.", 'yellow'))
            
            response = self.get_input(self._colorize("Would you like to reset your API key? (y/N): ", 'bright_cyan'))
            if response.lower() == 'y':
                self.reset_api_key()
                return True
        
        return False

    def ask_ai(self, question: str) -> str:
        """Ask AI a general question (not command-related) - Enhanced for rich markdown."""
        
        if not self.api_key or not self.model:
            return "Error: No API key configured or model not initialized"
        
        context = self._get_directory_context()
        
        prompt = textwrap.dedent(f"""You are a helpful AI assistant. Answer the user's question with rich markdown formatting.

        Use these markdown features extensively:
        - # Main headings and ## Subheadings
        - **Bold text** for key concepts
        - *Italic text* for emphasis
        - `inline code` for technical terms
        - ```language
          code blocks with syntax highlighting
          ```
        - > Blockquotes for important information
        - - Bullet points for lists
        - 1. Numbered lists for steps
        - [Links](https://example.com) when relevant
        - Tables when appropriate

        SYSTEM CONTEXT (for reference only):
        - OS: {self.system_info['os']}
        - Current Directory: {self.current_dir}
        
        QUESTION: "{question}"

        Provide a comprehensive, well-structured answer with excellent markdown formatting.""")

        try:
            response = self.model.generate_content(prompt)
            return response.text.strip()
        except Exception as e:
            error_msg = str(e)
            if self.handle_api_error(error_msg):
                # API key was reset, try again
                return self.ask_ai(question)
            return f"Sorry, I couldn't process your question: {error_msg}"

    def _setup_api_key(self):
        """Setup Gemini API key with interactive prompt if needed."""
        # Try to load from .env file first
        if self.env_file.exists():
            try:
                with open(self.env_file, 'r') as f:
                    for line in f:
                        line = line.strip()
                        if line.startswith('GEMINI='):
                            self.api_key = line.split('=', 1)[1].strip().strip('"\'')
                            break
            except Exception as e:
                print(f"Warning: Could not read .env file: {e}")
        
        # If no API key found, prompt user
        if not self.api_key:
            self._prompt_for_api_key()

    def _prompt_for_api_key(self):
        """Prompt user for Gemini API key and save it."""
        print(self._colorize('🔑 Gemini API Key Setup Required', 'bright_yellow'))
        print(self._colorize('=' * 45, 'bright_blue'))
        print("To use Commandor, you need a Gemini API key.")
        print("You can get one free at: https://makersuite.google.com/app/apikey")
        print()
        
        while True:
            try:
                api_key = input(self._colorize("Please enter your Gemini API key: ", 'bright_cyan')).strip()
                
                if not api_key:
                    print(self._colorize("❌ API key cannot be empty. Please try again.", 'red'))
                    continue
                
                # Test the API key
                print(self._colorize("🔍 Validating API key...", 'yellow'))
                
                try:
                    genai.configure(api_key=api_key)
                    test_model = genai.GenerativeModel("gemini-2.0-flash")
                    test_response = test_model.generate_content("Hello")
                    
                    if test_response:
                        self.api_key = api_key
                        self._save_api_key()
                        print(self._colorize("✅ API key validated and saved successfully!", 'bright_green'))
                        self.model = test_model
                        break
                    else:
                        print(self._colorize("❌ Invalid API key. Please check and try again.", 'red'))
                        
                except Exception as e:
                    print(self._colorize(f"❌ API key validation failed: {str(e)}", 'red'))
                    print(self._colorize("Please check your API key and try again.", 'yellow'))
                    
            except KeyboardInterrupt:
                print(f"\n{self._colorize('❌ Setup cancelled. Commandor requires an API key to function.', 'red')}")
                exit(1)

    def _save_api_key(self):
        """Save API key to .env file."""
        try:
            with open(self.env_file, 'w') as f:
                f.write(f'GEMINI={self.api_key}\n')
            
            # Set file permissions to be readable only by owner
            if os.name != 'nt':  # Not Windows
                os.chmod(self.env_file, 0o600)
                
        except Exception as e:
            print(f"Warning: Could not save API key: {e}")

    def _get_system_info(self) -> dict:
        """Gather system information for better context."""
        return {
            'os': platform.system(),
            'os_version': platform.version(),
            'architecture': platform.machine(),
            'python_version': platform.python_version(),
            'shell': os.environ.get('SHELL', 'unknown'),
            'user': os.environ.get('USER', os.environ.get('USERNAME', 'unknown'))
        }

    def _setup_readline(self):
        """Setup readline for better terminal input experience."""
        if not READLINE_AVAILABLE:
            return
        
        # Set up history file
        history_file = self.config_dir / 'history'
        
        try:
            # Load existing history
            if history_file.exists():
                readline.read_history_file(str(history_file))
            
            # Set history length
            readline.set_history_length(1000)
            
            # Enable tab completion for file paths
            readline.set_completer_delims(' \t\n`!@#$%^&*()=+[{]}\\|;:\'",<>?')
            readline.parse_and_bind("tab: complete")
            
            # Enable vi or emacs mode (emacs is default)
            readline.parse_and_bind("set editing-mode emacs")
            
            # Configure better line wrapping behavior
            readline.parse_and_bind("set horizontal-scroll-mode off")
            readline.parse_and_bind("set disable-completion off")
            
            # Custom key bindings
            readline.parse_and_bind("\\C-p: previous-history")  # Ctrl+P for previous
            readline.parse_and_bind("\\C-n: next-history")     # Ctrl+N for next
            
            # Save history on exit
            import atexit
            atexit.register(lambda: self._save_history(history_file))
            
        except Exception as e:
            print(f"Warning: Could not setup readline: {e}")

    def _save_history(self, history_file):
        """Save command history to file."""
        try:
            readline.write_history_file(str(history_file))
        except Exception as e:
            print(f"Warning: Could not save history: {e}")

    def _load_ask_history(self):
        """Load ask prompt history from file."""
        try:
            if self.ask_history_file.exists():
                with open(self.ask_history_file, 'r', encoding='utf-8') as f:
                    for line in f:
                        line = line.strip()
                        if line:
                            self.ask_history.append(line)
        except Exception as e:
            print(f"Warning: Could not load ask history: {e}")

    def _save_ask_history(self):
        """Save ask prompt history to file."""
        try:
            with open(self.ask_history_file, 'w', encoding='utf-8') as f:
                for prompt in self.ask_history:
                    f.write(f"{prompt}\n")
        except Exception as e:
            print(f"Warning: Could not save ask history: {e}")

    def show_ask_history(self):
        """Display ask prompt history."""
        if not self.ask_history:
            print(self._colorize("🤔 No ask history available", 'yellow'))
            return
        
        print(self._colorize(f"📝 Ask History ({len(self.ask_history)} total):", 'bold'))
        print(self._colorize('-' * 40, 'bright_magenta'))
        for i, prompt in enumerate(self.ask_history[-15:], 1):
            # Truncate long prompts for display
            display_prompt = prompt if len(prompt) <= 60 else prompt[:57] + "..."
            print(f"{self._colorize(f'{i:2d}.', 'bright_cyan')} {display_prompt}")
        
        if len(self.ask_history) > 15:
            print(f"{self._colorize(f'... and {len(self.ask_history) - 15} more', 'yellow')}")
        
        print(f"\n{self._colorize('💡 Tip:', 'bright_yellow')} Use '/ask-search <term>' to search your ask history")
    
    def search_ask_history(self, search_term: str):
        """Search ask history for a specific term."""
        if not self.ask_history:
            print(self._colorize("🤔 No ask history to search", 'yellow'))
            return
        
        search_term_lower = search_term.lower()
        matches = []
        
        for i, prompt in enumerate(self.ask_history):
            if search_term_lower in prompt.lower():
                matches.append((i + 1, prompt))
        
        if not matches:
            print(self._colorize(f"🔍 No matches found for '{search_term}'", 'yellow'))
            return
        
        print(self._colorize(f"🔍 Found {len(matches)} match(es) for '{search_term}':", 'bold'))
        print(self._colorize('-' * 50, 'bright_green'))
        for index, prompt in matches:
            # Highlight the search term in the result
            highlighted_prompt = prompt.replace(
                search_term, 
                self._colorize(search_term, 'bright_yellow')
            )
            print(f"{self._colorize(f'{index:2d}.', 'bright_cyan')} {highlighted_prompt}")

    def _colorize(self, text: str, color: str) -> str:
        """Apply color to text."""
        try:
            if hasattr(self, 'colors') and self.colors:
                return f"{self.colors.get(color, '')}{text}{self.colors['reset']}"
            else:
                return text
        except Exception:
            return text

    def _display_colorful_logo(self):
        """Display colorful Commandor logo."""
        logo = f"""
{self._colorize('╔═══════════════════════════════════════════════════════════════╗', 'bright_cyan')}
{self._colorize('║', 'bright_cyan')}  {self._colorize('█▀▀ █▀█ █▀▄▀█ █▀▄▀█ █▀█ █▄░█ █▀▄ █▀█ █▀█', 'bright_magenta')}  {self._colorize('║', 'bright_cyan')}
{self._colorize('║', 'bright_cyan')}  {self._colorize('█▄▄ █▄█ █░▀░█ █░▀░█ █▄█ █░▀█ █▄▀ █▄█ █▀▄', 'bright_blue')}  {self._colorize('║', 'bright_cyan')}
{self._colorize('║', 'bright_cyan')}                                                             {self._colorize('║', 'bright_cyan')}
{self._colorize('║', 'bright_cyan')}    {self._colorize('🤖 Your AI-Powered Terminal Assistant 🤖', 'bright_yellow')}        {self._colorize('║', 'bright_cyan')}
{self._colorize('║', 'bright_cyan')}           {self._colorize('Speak naturally, execute powerfully!', 'bright_green')}       {self._colorize('║', 'bright_cyan')}
{self._colorize('╚═══════════════════════════════════════════════════════════════╝', 'bright_cyan')}
        """
        print(logo)

    def _get_directory_context(self) -> str:
        """Get context about the current directory."""
        try:
            files = []
            dirs = []
            
            for item in sorted(self.current_dir.iterdir())[:10]:
                if item.is_file():
                    files.append(item.name)
                elif item.is_dir():
                    dirs.append(item.name)
            
            context = f"Current directory: {self.current_dir}\n"
            if dirs:
                context += f"Directories: {', '.join(dirs[:5])}\n"
            if files:
                context += f"Files: {', '.join(files[:5])}\n"
                
            return context
        except PermissionError:
            return f"Current directory: {self.current_dir} (limited access)\n"

    def _get_recent_commands(self) -> str:
        """Get recent command history for context."""
        if not self.command_history:
            return ""
        
        recent = self.command_history[-3:]
        return "Recent commands:\n" + "\n".join([f"  {cmd}" for cmd in recent]) + "\n"

    def get_ai_command(self, instruction: str) -> str:
        """Convert natural language instruction to shell command using AI."""
        if not self.api_key or not self.model:
            return "# Error: No API key configured or model not initialized"
        
        context = self._get_directory_context()
        recent_commands = self._get_recent_commands()
        
        prompt = f"""You are an expert system administrator helping convert natural language to shell commands.

        SYSTEM INFORMATION:
        - OS: {self.system_info['os']} {self.system_info['os_version']}
        - Architecture: {self.system_info['architecture']}
        - Shell: {self.system_info['shell']}
        - User: {self.system_info['user']}

        CURRENT CONTEXT:
        {context}

        {recent_commands}

        INSTRUCTION: "{instruction}"

        RULES:
        1. Reply with ONLY the shell command, no explanations
        2. Use commands appropriate for {self.system_info['os']}
        3. Consider the current directory context
        4. Use safe commands (avoid destructive operations without explicit confirmation)
        5. For file operations, use relative paths when possible
        6. If the instruction is ambiguous, choose the most common interpretation

        Command:"""

        try:
            response = self.model.generate_content(prompt)
            command = response.text.strip()
            
            if command.startswith('```'):
                lines = command.split('\n')
                command = '\n'.join(lines[1:-1]) if len(lines) > 2 else lines[1]
            
            return command.strip()
        except Exception as e:
            error_msg = str(e)
            if self.handle_api_error(error_msg):
                # API key was reset, try again
                return self.get_ai_command(instruction)
            return f"# Error getting AI response: {error_msg}"

    def execute_command(self, command: str) -> Tuple[bool, str, str]:
        """Execute a shell command and return success status and output."""
        try:
            if command.strip().startswith('cd '):
                return self._handle_cd_command(command)
            
            result = subprocess.run(
                command,
                shell=True,
                cwd=str(self.current_dir),
                capture_output=True,
                text=True,
                timeout=30
            )
            
            success = result.returncode == 0
            return success, result.stdout, result.stderr
            
        except subprocess.TimeoutExpired:
            return False, "", "Command timed out after 30 seconds"
        except Exception as e:
            return False, "", str(e)

    def _handle_cd_command(self, command: str) -> Tuple[bool, str, str]:
        """Handle cd commands to update current directory."""
        parts = command.strip().split(maxsplit=1)
        
        if len(parts) == 1:
            target = Path.home()
        else:
            path = parts[1].strip('\'"')
            target = Path(path)
            
            if not target.is_absolute():
                target = self.current_dir / target
        
        try:
            resolved_target = target.resolve()
            if resolved_target.exists() and resolved_target.is_dir():
                os.chdir(resolved_target)
                self.current_dir = resolved_target
                return True, f"Changed directory to {resolved_target}", ""
            else:
                return False, "", f"Directory not found: {target}"
        except (OSError, PermissionError) as e:
            return False, "", f"Cannot access directory: {e}"

    def display_output(self, success: bool, stdout: str, stderr: str):
        """Display command output with appropriate formatting."""
        if stdout:
            print(stdout, end='')
        
        if stderr:
            print(self._colorize(stderr, 'red'), end='')
        
        if not success and not stderr:
            print(self._colorize("Command failed", 'red'))
        
        # Always add newline after any command output to ensure prompt appears on new line
        # This prevents text overlapping issues when the next prompt is displayed
        print()

    def get_prompt(self) -> str:
        """Generate the terminal prompt."""
        # Show last 2 directories for better context
        path_parts = self.current_dir.parts
        if len(path_parts) > 2:
            dir_display = f".../{'/'.join(path_parts[-2:])}"
        else:
            dir_display = str(self.current_dir)
        
        if len(dir_display) > 30:
            dir_display = "..." + dir_display[-27:]
        
        return f"{self._colorize('Commandor', 'bright_cyan')} {self._colorize(f'[{dir_display}]', 'bright_blue')} {self._colorize('# ', 'bright_yellow')}"

    def show_help(self):
        """Display help information."""
        help_text = f"""
        {self._colorize('🚀 Commandor Help Guide 🚀', 'bold')}
        {self._colorize('=' * 60, 'bright_blue')}

        {self._colorize('Special Commands:', 'bright_green')}
        {self._colorize('/ai <instruction>', 'bright_cyan')}  - Convert natural language to shell command
        {self._colorize('/ask <question>', 'bright_magenta')}   - Ask AI any question directly
        {self._colorize('/help', 'yellow')}             - Show this help message
        {self._colorize('/info', 'yellow')}             - Show system information
        {self._colorize('/history', 'yellow')}          - Show recent command history
        {self._colorize('/ask-history', 'yellow')}      - Show your question history
        {self._colorize('/ask-search <term>', 'yellow')} - Search your question history
        {self._colorize('/clear', 'yellow')}            - Clear the screen
        {self._colorize('/config', 'yellow')}           - Show configuration info
        {self._colorize('/reset-api', 'yellow')}        - Reset and reconfigure API key
        {self._colorize('/test-api', 'yellow')}         - Test current API key
        {self._colorize('exit', 'red')} or {self._colorize('Ctrl+C', 'red')}       - Exit the terminal

        {self._colorize('AI Command Examples:', 'bright_yellow')}
        {self._colorize('/ai', 'bright_cyan')} list all python files
        {self._colorize('/ai', 'bright_cyan')} create a new directory called projects
        {self._colorize('/ai', 'bright_cyan')} find files larger than 100MB
        {self._colorize('/ai', 'bright_cyan')} show disk usage
        {self._colorize('/ai', 'bright_cyan')} install package using pip

        {self._colorize('Ask AI Examples:', 'bright_yellow')}
        {self._colorize('/ask', 'bright_magenta')} What is the difference between Python and JavaScript?
        {self._colorize('/ask', 'bright_magenta')} How do I optimize my code for better performance?
        {self._colorize('/ask', 'bright_magenta')} Explain machine learning concepts
        {self._colorize('/ask', 'bright_magenta')} What are best practices for Git workflow?

        {self._colorize('API Management:', 'bright_yellow')}
        {self._colorize('/reset-api', 'yellow')}         - Change your API key
        {self._colorize('/test-api', 'yellow')}          - Verify API key is working

        {self._colorize('Ask History Examples:', 'bright_yellow')}
        {self._colorize('/ask-history', 'yellow')}       - View your recent questions
        {self._colorize('/ask-search python', 'yellow')} - Find questions about Python
        {self._colorize('/ask-search debug', 'yellow')}  - Find debugging-related questions

        {self._colorize('💡 Regular shell commands work too!', 'bright_green')}
        """
        print(help_text)

    def show_info(self):
        """Display system information."""
        info_text = f"""
        {self._colorize('🖥️  System Information', 'bold')}
        {self._colorize('=' * 35, 'bright_blue')}
        {self._colorize('OS:', 'bright_cyan')} {self.system_info['os']} {self.system_info['os_version']}
        {self._colorize('Architecture:', 'bright_cyan')} {self.system_info['architecture']}
        {self._colorize('Python:', 'bright_cyan')} {self.system_info['python_version']}
        {self._colorize('Shell:', 'bright_cyan')} {self.system_info['shell']}
        {self._colorize('User:', 'bright_cyan')} {self.system_info['user']}
        {self._colorize('Current Directory:', 'bright_cyan')} {self.current_dir}
        """
        print(info_text)

    def show_config(self):
        """Display configuration information."""
        config_text = f"""
        {self._colorize('⚙️  Configuration', 'bold')}
        {self._colorize('=' * 25, 'bright_blue')}
        {self._colorize('Config Directory:', 'bright_cyan')} {self.config_dir}
        {self._colorize('API Key Status:', 'bright_cyan')} {'✅ Configured' if self.api_key else '❌ Not configured'}
        {self._colorize('Model Status:', 'bright_cyan')} {'✅ Initialized' if self.model else '❌ Not initialized'}
        {self._colorize('Readline Support:', 'bright_cyan')} {'✅ Available' if READLINE_AVAILABLE else '❌ Not available'}
        {self._colorize('API Key File:', 'bright_cyan')} {self.env_file}
        {self._colorize('Ask History File:', 'bright_cyan')} {self.ask_history_file}
        """
        print(config_text)

    def show_history(self):
        """Display recent command history."""
        if not self.command_history:
            print(self._colorize("📝 No command history available", 'yellow'))
            return
        
        print(self._colorize("📚 Recent Commands:", 'bold'))
        print(self._colorize('-' * 25, 'bright_blue'))
        for i, cmd in enumerate(self.command_history[-10:], 1):
            print(f"{self._colorize(f'{i:2d}.', 'bright_cyan')} {cmd}")

    def add_to_history(self, command: str):
        """Add command to history."""
        if command and command not in ['exit', '/help', '/info', '/history', '/clear', '/config', '/ask-history', '/reset-api', '/test-api']:
            self.command_history.append(command)
            if len(self.command_history) > self.max_history:
                self.command_history.pop(0)
            
            if READLINE_AVAILABLE:
                readline.add_history(command)

    def add_to_ask_history(self, prompt: str):
        """Add ask prompt to ask history."""
        if prompt and prompt.strip():
            # Avoid duplicates if the same question was asked recently
            if not self.ask_history or self.ask_history[-1] != prompt:
                self.ask_history.append(prompt)
                if len(self.ask_history) > self.max_ask_history:
                    self.ask_history.pop(0)
                # Save to file immediately for persistence
                self._save_ask_history()

    def get_input(self, prompt: str) -> str:
        """Get user input with proper readline support and line wrapping prevention."""
        try:
            # Get input normally - the real fix is ensuring clean line termination elsewhere
            user_input = input(prompt).strip()
            return user_input
        except EOFError:
            raise KeyboardInterrupt
        
    def run(self):
        """Main terminal loop."""
        if not self.api_key or not self.model:
            print(self._colorize("❌ Cannot start Commandor without API key or model initialization.", 'red'))
            return
        
        self._display_colorful_logo()
        
        if READLINE_AVAILABLE:
            print(f"✨ {self._colorize('Enhanced input mode active!', 'bright_green')}")
        else:
            print("⚠️  Basic input mode (install 'readline' or 'pyreadline3' for better experience)")
        
        print(f"Type {self._colorize('/help', 'bright_cyan')} for commands or {self._colorize('Ctrl+C', 'bright_yellow')} to exit.")
        print(f"Use {self._colorize('/ai', 'bright_cyan')} for commands or {self._colorize('/ask', 'bright_magenta')} for questions!")
        print()

        while True:
            try:
                user_input = self.get_input(self.get_prompt())
                
                if not user_input:
                    continue
                
                if user_input == 'exit':
                    break
                elif user_input == '/help':
                    self.show_help()
                    continue
                elif user_input == '/info':
                    self.show_info()
                    continue
                elif user_input == '/config':
                    self.show_config()
                    continue
                elif user_input == '/history':
                    self.show_history()
                    continue
                elif user_input == '/ask-history':
                    self.show_ask_history()
                    continue
                elif user_input.startswith('/ask-search '):
                    search_term = user_input[12:].strip()
                    if not search_term:
                        print(self._colorize("🔍 Please provide a search term after /ask-search", 'yellow'))
                        continue
                    self.search_ask_history(search_term)
                    continue
                elif user_input == '/clear':
                    os.system('clear' if os.name != 'nt' else 'cls')
                    continue
                elif user_input == '/reset-api':
                    self.reset_api_key()
                    continue
                elif user_input == '/test-api':
                    if self.test_api_key():
                        print(self._colorize("✅ API key is working correctly!", 'bright_green'))
                    else:
                        print(self._colorize("❌ API key test failed", 'red'))
                    continue
                
                elif user_input.startswith('/ask '):
                    question = user_input[5:].strip()
                    if not question:
                        print(self._colorize("❓ Please provide a question after /ask", 'yellow'))
                        continue
                    
                    # Check if API key and model are still valid
                    if not self.api_key or not self.model:
                        print(self._colorize("❌ API key or model not available. Use /reset-api to reconfigure.", 'red'))
                        continue
                    
                    # Add question to ask history before processing
                    self.add_to_ask_history(question)
                    
                    print(self._colorize("🤔 Thinking...", 'yellow'), flush=True)
                    ai_response = self.ask_ai(question)
                    
                    # Check if there was an API error that couldn't be resolved
                    if ai_response.startswith("Error:") or ai_response.startswith("Sorry,"):
                        print(self._colorize(ai_response, 'red'))
                    else:
                        # Use the enhanced display instead of simple print
                        self._display_ai_response(ai_response, "AI Response")
                    
                    self.add_to_history(f"/ask {question}")
                    continue
                
                elif user_input.startswith('/ai '):
                    instruction = user_input[4:].strip()
                    if not instruction:
                        print(self._colorize("💡 Please provide an instruction after /ai", 'yellow'))
                        continue
                    
                    # Check if API key and model are still valid
                    if not self.api_key or not self.model:
                        print(self._colorize("❌ API key or model not available. Use /reset-api to reconfigure.", 'red'))
                        continue
                    
                    print(self._colorize("🧠 Generating command...", 'yellow'), flush=True)
                    ai_command = self.get_ai_command(instruction)
                    
                    # Check if there was an API error
                    if ai_command.startswith("# Error"):
                        print(self._colorize(ai_command, 'red'))
                        continue
                    
                    print(f"{self._colorize('🤖 AI →', 'bright_green')} {self._colorize(ai_command, 'bright_blue')}")
                    
                    # Safety check for dangerous commands
                    dangerous_patterns = ['rm -rf', 'sudo rm', 'format', 'mkfs', '> /dev/', 'dd if=']
                    if any(pattern in ai_command.lower() for pattern in dangerous_patterns):
                        confirm = self.get_input(self._colorize("⚠️  This command looks dangerous. Execute? (y/N): ", 'bright_yellow'))
                        if confirm.lower() != 'y':
                            print(self._colorize("❌ Command cancelled", 'yellow'))
                            continue
                    
                    success, stdout, stderr = self.execute_command(ai_command)
                    self.display_output(success, stdout, stderr)
                    self.add_to_history(f"/ai {instruction} → {ai_command}")
                
                else:
                    # Regular shell command
                    success, stdout, stderr = self.execute_command(user_input)
                    self.display_output(success, stdout, stderr)
                    self.add_to_history(user_input)
                    
            except KeyboardInterrupt:
                print(f"\n{self._colorize('👋 Goodbye! Thanks for using Commandor!', 'bright_green')}")
                break
            except EOFError:
                print(f"\n{self._colorize('👋 Goodbye! Thanks for using Commandor!', 'bright_green')}")
                break
            except Exception as e:
                print(f"💥 Unexpected error: {e}")
                # Add some debugging info
                import traceback
                print(f"Error details: {traceback.format_exc()}")
                # Offer to reset API if it might be an API-related error
                if "api" in str(e).lower() or "model" in str(e).lower():
                    response = self.get_input(self._colorize("This might be an API issue. Reset API key? (y/N): ", 'bright_cyan'))
                    if response.lower() == 'y':
                        self.reset_api_key()